@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.dir.c,v 3.56 2001/01/29 01:28:02 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.dir.c,v 3.63 2004/05/10 19:12:37 christos Exp $ */
 /*
  * sh.dir.c: Directory manipulation functions
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.dir.c,v 3.56 2001/01/29 01:28:02 christos Exp $")
+RCSID("$Id: sh.dir.c,v 3.63 2004/05/10 19:12:37 christos Exp $")
 
 /*
  * C Shell - directory management
@@ -268,7 +264,7 @@ printdirs(dflag)
 	}
 	if (user) 
 	    xprintf("~%S", user);
-	xprintf("%S%c", s, (dflag & DIR_VERT) ? '\n' : ' ');
+	xprintf("\045S%c", s, (dflag & DIR_VERT) ? '\n' : ' ');
     } while ((dp = dp->di_prev) != dcwd);
     if (!(dflag & DIR_VERT))
 	xputchar('\n');
@@ -280,7 +276,7 @@ dtildepr(dir)
 {
     Char* user;
     if ((user = getusername(&dir)) != NULL)
-	xprintf("~%S%S", user, dir);
+	xprintf("~\045S%S", user, dir);
     else
 	xprintf("%S", dir);
 }
@@ -323,6 +319,7 @@ dnormalize(cp, exp)
     if (exp) {
  	int     dotdot = 0;
 	Char   *dp, *cwd, *start = cp, buf[MAXPATHLEN];
+	struct stat sb;
 # ifdef apollo
 	bool slashslash;
 # endif /* apollo */
@@ -339,6 +336,13 @@ dnormalize(cp, exp)
         if (dotdot == 0)
 	    return (Strsave(cp));
 
+	/*
+	 * If the path doesn't exist, we are done too.
+	 */
+	if (lstat(short2str(cp), &sb) != 0 && errno == ENOENT)
+	    return (Strsave(cp));
+	
+
 	cwd = (Char *) xmalloc((size_t) (((int) Strlen(dcwd->di_name) + 3) *
 					   sizeof(Char)));
 	(void) Strcpy(cwd, dcwd->di_name);
@@ -415,6 +419,18 @@ dnormalize(cp, exp)
 	        if ((TRM(cwd[(dotdot = (int) Strlen(cwd)) - 1])) == '/')
 		    cwd[--dotdot] = '\0';
 	    }
+	    /* Reduction of ".." following the stuff we collected in buf
+	     * only makes sense if the directory item in buf really exists.
+	     * Avoid reduction of "-I../.." (typical compiler call) to ""
+	     * or "/usr/nonexistant/../bin" to "/usr/bin":
+	     */
+	    if (cwd[0]) {
+	        struct stat exists;
+		if (0 != stat(short2str(cwd), &exists)) {
+		    xfree((ptr_t) cwd);
+		    return Strsave(start);
+		}
+	    }
 	    if (!*cp)
 	        break;
 	}
@@ -513,8 +529,13 @@ dgoto(cp)
     else
 	dp = cp;
 
-#ifdef WINNT_NATIVE
+#if defined(WINNT_NATIVE)
     cp = SAVE(getcwd(NULL, 0));
+#elif defined(__CYGWIN__)
+    if (ABSOLUTEP(cp) && cp[1] == ':') /* Only DOS paths are treated that way */
+    	cp = SAVE(getcwd(NULL, 0));
+    else
+    	cp = dcanon(cp, dp);
 #else /* !WINNT_NATIVE */
     cp = dcanon(cp, dp);
 #endif /* WINNT_NATIVE */
@@ -570,7 +591,7 @@ dfollow(cp)
     }
 
     if (cp[0] != '/' && !prefix(STRdotsl, cp) && !prefix(STRdotdotsl, cp)
-	&& (c = adrof(STRcdpath))) {
+	&& (c = adrof(STRcdpath)) && c->vec != NULL) {
 	Char  **cdp;
 	register Char *p;
 	Char    buf[MAXPATHLEN];
@@ -912,7 +933,7 @@ dcanon(cp, p)
 #ifdef S_IFLNK			/* if we have symlinks */
 	    if (sp != cp && /* symlinks != SYM_IGNORE && */
 		(cc = readlink(short2str(cp), tlink,
-			       sizeof tlink)) >= 0) {
+			       sizeof(tlink) - 1)) >= 0) {
 		tlink[cc] = '\0';
 		(void) Strncpy(link, str2short(tlink),
 		    sizeof(link) / sizeof(Char));
@@ -1003,7 +1024,7 @@ dcanon(cp, p)
 #ifdef S_IFLNK			/* if we have symlinks */
 	    if (sp != cp && symlinks == SYM_CHASE &&
 		(cc = readlink(short2str(cp), tlink,
-			       sizeof tlink)) >= 0) {
+			       sizeof(tlink) - 1)) >= 0) {
 		tlink[cc] = '\0';
 		(void) Strncpy(link, str2short(tlink),
 		    sizeof(link) / sizeof(Char));
@@ -1201,7 +1222,7 @@ dsetstack()
     struct varent *vp;
     struct directory *dn, *dp;
 
-    if ((vp = adrof(STRdirstack)) == NULL)
+    if ((vp = adrof(STRdirstack)) == NULL || vp->vec == NULL)
 	return;
 
     /* Free the whole stack */
