@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.exec.c,v 3.57 2002/11/21 20:02:01 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.exec.c,v 3.58 2003/03/12 19:14:51 christos Exp $ */
 /*
  * sh.exec.c: Search, find, and execute a command!
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.exec.c,v 3.57 2002/11/21 20:02:01 christos Exp $")
+RCSID("$Id: sh.exec.c,v 3.58 2003/03/12 19:14:51 christos Exp $")
 
 #include "tc.h"
 #include "tw.h"
@@ -440,6 +440,9 @@ texec(sf, st)
 	 */
 	if ((fd = open(f, O_RDONLY|O_LARGEFILE)) != -1) {
 	    int nread;
+#ifdef O_TEXT
+	    setmode(fd, O_TEXT);
+#endif
 	    if ((nread = read(fd, (char *) pref, 2)) == 2) {
 		if (!Isprint(pref[0]) && (pref[0] != '\n' && pref[0] != '\t')) {
 		    (void) close(fd);
@@ -763,10 +766,15 @@ dohash(vv, c)
 	     */
 	    {
 		size_t	ext = strlen(dp->d_name) - 4;
-		if ((ext > 0) && (strcmp(&dp->d_name[ext], ".exe") == 0 ||
-				  strcmp(&dp->d_name[ext], ".bat") == 0 ||
-				  strcmp(&dp->d_name[ext], ".com") == 0))
-		    dp->d_name[ext] = '\0';
+		if ((ext > 0) && (strcasecmp(&dp->d_name[ext], ".exe") == 0 ||
+				  strcasecmp(&dp->d_name[ext], ".bat") == 0 ||
+				  strcasecmp(&dp->d_name[ext], ".com") == 0))
+		    {
+			dp->d_name[ext] = '\0';
+#if defined(__CYGWIN__)
+			strlwr(dp->d_name);
+#endif /* __CYGWIN__ */
+		    }
 	    }
 #endif /* _UWIN || __CYGWIN__ */
 # ifdef FASTHASH
