@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.glob.c,v 3.48 2001/01/04 18:51:57 christos Exp $ */
+/* $Header: /src/pub/tcsh/sh.glob.c,v 3.55 2004/03/21 16:48:14 christos Exp $ */
 /*
  * sh.glob.c: Regular expression expansion
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,9 +32,10 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.glob.c,v 3.48 2001/01/04 18:51:57 christos Exp $")
+RCSID("$Id: sh.glob.c,v 3.55 2004/03/21 16:48:14 christos Exp $")
 
 #include "tc.h"
+#include "tw.h"
 
 #include "glob.h"
 
@@ -82,7 +79,6 @@ static	Char	**libglob	__P((Char **));
 static	Char	**globexpand	__P((Char **));
 static	int	  globbrace	__P((Char *, Char *, Char ***));
 static  void	  expbrace	__P((Char ***, Char ***, int));
-static  int	  pmatch	__P((Char *, Char *, Char **));
 static	void	  pword		__P((int));
 static	void	  psave		__P((int));
 static	void	  backeval	__P((Char *, bool));
@@ -755,7 +751,7 @@ dobackp(cp, literal)
     pnleft = LONGBSIZE - 4;
     for (;;) {
 #if defined(DSPMBYTE)
-	for (lp = cp;; lp++) {
+	for (lp = cp;; lp++) { /* } */
 	    if (*lp == '`' &&
 		(lp-1 < cp || !Ismbyte2(*lp) || !Ismbyte1(*(lp-1)))) {
 		break;
@@ -834,6 +830,7 @@ backeval(cp, literal)
 	(void) dmove(pvec[1], 1);
 	(void) dmove(SHDIAG,  2);
 	initdesc();
+	closem();
 	/*
 	 * Bugfix for nested backquotes by Michael Greim <greim@sbsvax.UUCP>,
 	 * posted to comp.bugs.4bsd 12 Sep. 1989.
@@ -842,8 +839,11 @@ backeval(cp, literal)
 	    blkfree(pargv), pargv = 0, pargsiz = 0;
 	/* mg, 21.dec.88 */
 	arginp = cp;
-	while (*cp)
-	    *cp++ &= TRIM;
+	for (arginp = cp; *cp; cp++) {
+	    *cp &= TRIM;
+	    if (*cp == '\n' || *cp == '\r')
+		*cp = ' ';
+	}
 
         /*
 	 * In the child ``forget'' everything about current aliases or
@@ -871,7 +871,7 @@ backeval(cp, literal)
 #ifdef SIGTTOU
 	(void) sigignore(SIGTTOU);
 #endif
-	execute(t, -1, NULL, NULL);
+	execute(t, -1, NULL, NULL, TRUE);
 	exitstat();
     }
     xfree((ptr_t) cp);
@@ -987,12 +987,12 @@ Gnmatch(string, pattern, endstr)
     if (endstr == NULL)
 	/* Exact matches only */
 	for (p = blk; *p; p++) 
-	    gres |= pmatch(string, *p, &tstring) == 2 ? 1 : 0;
+	    gres |= t_pmatch(string, *p, &tstring, 0) == 2 ? 1 : 0;
     else {
 	/* partial matches */
 	int minc = 0x7fffffff;
 	for (p = blk; *p; p++) 
-	    if (pmatch(string, *p, &tstring) != 0) {
+	    if (t_pmatch(string, *p, &tstring, 0) != 0) {
 		int t = (int) (tstring - string);
 		gres |= 1;
 		if (minc == -1 || minc > t)
@@ -1005,17 +1005,18 @@ Gnmatch(string, pattern, endstr)
     return(gres == gpol);
 } 
 
-/* pmatch():
+/* t_pmatch():
  *	Return 2 on exact match, 	
  *	Return 1 on substring match.
  *	Return 0 on no match.
  *	*estr will point to the end of the longest exact or substring match.
  */
-static int
-pmatch(string, pattern, estr)
-    register Char *string, *pattern, **estr;
+int
+t_pmatch(string, pattern, estr, cs)
+    Char *string, *pattern, **estr;
+    int cs;
 {
-    register Char stringc, patternc;
+    Char    stringc, patternc;
     int     match, negate_range;
     Char    rangec, *oestr, *pestr;
 
@@ -1044,7 +1045,7 @@ pmatch(string, pattern, estr)
 	    pestr = NULL;
 
 	    do {
-		switch(pmatch(string, pattern, estr)) {
+		switch(t_pmatch(string, pattern, estr, cs)) {
 		case 0:
 		    break;
 		case 1:
@@ -1078,8 +1079,8 @@ pmatch(string, pattern, estr)
 		if (match)
 		    continue;
 		if (rangec == '-' && *(pattern-2) != '[' && *pattern  != ']') {
-		    match = (globcharcoll(stringc, *pattern & TRIM) <= 0 &&
-		    globcharcoll(*(pattern-2) & TRIM, stringc) <= 0);
+		    match = (globcharcoll(stringc, *pattern & TRIM, cs) <= 0 &&
+		    globcharcoll(*(pattern-2) & TRIM, stringc, cs) <= 0);
 		    pattern++;
 		}
 		else 
@@ -1125,7 +1126,7 @@ Gcat(s1, s2)
 	continue;
 }
 
-#ifdef FILEC
+#if defined(FILEC) && defined(TIOCSTI)
 int
 sortscmp(a, b)
     register Char **a, **b;
