@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/tc.decls.h,v 3.49 2001/08/06 23:57:33 christos Exp $ */
+/* $Header: /src/pub/tcsh/tc.decls.h,v 3.52 2004/02/21 20:34:25 christos Exp $ */
 /*
  * tc.decls.h: Function declarations from all the tcsh modules
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -145,6 +141,10 @@ extern	void 		  dodmmode	__P((Char **, struct command *));
 extern	void		  douniverse	__P((Char **, struct command *));
 #endif /* masscomp */
 
+#if defined(_OSD_POSIX) /* BS2000 */
+extern	void		  dobs2cmd	__P((Char **, struct command *));
+#endif /* _OSD_POSIX */
+
 #if defined(hcx)
 extern	void		  doatt		__P((Char **, struct command *));
 extern	void		  doucb		__P((Char **, struct command *));
@@ -303,6 +303,7 @@ extern	size_t		  s_strlen	__P((const Char *));
 extern	int		  s_strcmp	__P((const Char *, const Char *));
 extern	int		  s_strncmp	__P((const Char *, const Char *, 
 					     size_t));
+extern	int		  s_strcasecmp	__P((const Char *, const Char *));
 extern	Char		 *s_strsave	__P((const Char *));
 extern	Char		 *s_strend	__P((const Char *));
 extern	Char		 *s_strstr	__P((const Char *, const Char *));
